indexing
	description: "Representation of a month w/r/t a calendar"
	license: "Eiffel Forum License v2 - See license.txt"
	source: "{
		Amalasoft Corporation
		273 Harwood Avenue
		Littleton, MA 01460 USA
		}"
	status: "Copyright 1995-2008, Amalasoft"
	date: "$Date: 2008/04/26 $"
	revision: "$Revision: 002$"

class AEL_DT_CALENDAR_MONTH

inherit
	ANY
		redefine
			out
		end

create
	make

feature {NONE} -- Creation

	make (m: INTEGER; y: AEL_DT_CALENDAR_YEAR) is
		require
			valid_month: m > 0 and then m <= 12
			year_exists: y /= Void
		do
			month_number := m
			year := y
			set_first_day_position_in_week
			set_number_of_days
		end

feature -- Status

	name: STRING is
		do
			Result := cal_const.month_long_names.item (month_number)
		end

	month_number: INTEGER
	year: AEL_DT_CALENDAR_YEAR

	is_julian: BOOLEAN is
		do
			Result := year.is_julian
		end

	first_day_position_in_week: INTEGER
		-- On which day of the week does the
		-- first day of the month fall?

	first_day_in_week (v: INTEGER): INTEGER is
		-- For a given week in this month, what is the 1-based day
		-- that begins the week?  If the week does not begin with
		-- a day from this month, then the result is negative.
		require
			valid_week: is_valid_week_number (v)
		local
			fd: INTEGER
		do
			fd := first_day_position_in_week
			if v = 1 then
				if fd = 1 then
					Result := 1
				else
					Result := -1
				end
			elseif v > number_of_weeks then
				Result := -1
			else
				fd := 9 - fd -- second week, first day
				Result := (7 * (v - 2)) + fd
			end
		end

	first_day_of_week: INTEGER is
		-- Which day of week is first? Sunday or Monday?
		-- Is 1 if Sunday, 2 if Monday
		do
			Result := private_first_day_of_week + 1
		end

	number_of_days: INTEGER

	number_of_weeks: INTEGER is
		local
			pos: INTEGER
			dt: INTEGER
			tw: INTEGER
		do
			pos := first_day_position_in_week
			dt := number_of_days
			tw := dt - (7 - pos)
			Result := ((tw / 7) + 1).ceiling
		ensure
			result_valid: Result >= 4 and Result <= 6
		end

feature -- String representation

	out: STRING is
		local
			idx, midx: INTEGER
			lim: INTEGER
			ij: BOOLEAN
			tstr: STRING
			fd, tl: INTEGER
			pf: AEL_PRINTF
			fmt: STRING
		do
			lim := number_of_days
			ij := is_julian

			create Result.make (256)

			if ij then
				tl := 28
			else
				tl := 21
			end
			create tstr.make (tl)
			tstr.append (cal_const.month_long_names.item (month_number))
			tstr.append (" ")
			tstr.append_integer (year.year_number)
			create pf
			fmt := "%%="
			fmt.append_integer (tl)
			fmt.extend ('s')
			Result.append (pf.aprintf (fmt, << " " >>))
			Result.prune_all_trailing (' ')
			Result.extend ('%N')
			Result.append (day_labels)
			Result.prune_all_trailing (' ')
			Result.extend ('%N')

			fd := first_day_position_in_week
			from idx := 1
			until idx >= fd
			loop
				Result.append (spacer_string)
				idx := idx + 1
			end

			from midx := 1
			until midx > lim
			loop
				if idx > 7 then
					Result.prune_all_trailing (' ')
					Result.extend ('%N')
					idx := 1
				end
				Result.append (cal_routines.ascii_day (midx, ij))
				midx := midx + 1
				idx := idx + 1
			end
			Result.prune_all_trailing (' ')
		end

	week_out (wn: INTEGER): STRING is
		require
			valid_week: is_valid_week_number (wn)
		local
			idx, midx: INTEGER
			lim: INTEGER
			ij: BOOLEAN
			fd, tl: INTEGER
		do
			lim := number_of_days
			ij := is_julian

			create Result.make (256)

			if wn = 0 then
				Result.append (day_labels)
			elseif wn = 1 then
				fd := first_day_position_in_week
				from idx := 1
				until idx >= fd
				loop
					Result.append (spacer_string)
					idx := idx + 1
				end
				from midx := 1
				until idx > 7
				loop
					Result.append (cal_routines.ascii_day (midx, ij))
					idx := idx + 1
					midx := midx + 1
				end
			else
				-- Output the given week
				fd := first_day_in_week (wn)
				from idx := 0
				until idx = 7
				loop
					if fd < 0 then
						tl := cal_const.K_space
					else
						tl := fd + idx
					end
					if tl > lim then
						tl := cal_const.K_space
					end
					Result.append (cal_routines.ascii_day (tl, is_julian))
					idx := idx + 1
				end
			end
		end

	day_labels: STRING is
		do
			Result := private_day_labels
			if Result = Void then
				if is_julian then
					Result := "  S   M  Tu   W  Th   F   S "
				else
					Result := " S  M Tu  W Th  F  S "
				end
			end
		end

	spacer_string: STRING is
		-- Spaces equivalent of 2 digits and a blank
		do
			if is_julian then
				Result := "    "
			else
				Result := "   "
			end
		end

	is_valid_week_number (wn: INTEGER): BOOLEAN is
		do
			Result := cal_routines.is_valid_week_number (wn)
		end

feature -- Status setting

	set_first_day_position_in_week is
		do
			first_day_position_in_week := cal_routines.day_in_week (1, month_number, year.year_number)
		end

	set_monday_first is
		do
			private_first_day_of_week := 1
		end

	set_sunday_first is
		do
			private_first_day_of_week := 0
		end

	set_number_of_days is
		do
			number_of_days := cal_routines.num_days_in_month (month_number, year.is_leap_year)
		end

	set_day_labels (v: like day_labels) is
		do
			private_day_labels := v
		end

feature {NONE} -- Implementation and constants

	private_day_labels: like day_labels
	private_first_day_of_week: INTEGER

	cal_const: AEL_DATE_TIME_CONSTANTS is
		once
			create Result
		end

	cal_routines: AEL_DATE_TIME_ROUTINES is
		once
			create Result
		end

end -- class AEL_DT_CALENDAR_MONTH
